=== modified file 'NEWS'
--- NEWS	2009-05-13 14:30:32 +0000
+++ NEWS	2009-06-05 01:59:11 +0000
@@ -4,6 +4,8 @@
 dev [future]
 ---------------
 
+    - Add bug links in revision informations (Alexandre Garnier, #314052)
+
     - Make sure that binary files aren't annotated. (Martin Albisetti,
       #258848)
 
@@ -110,6 +112,18 @@
     - Fix internal server errors when using start-loggerhead. (Matt
       Nordhoff, #375948)
 
+    - Fix annotating non-UTF-8 files when Pygments is disabled. (Matt
+      Nordhoff, #376957)
+
+    - Fix 'bzr serve --http' errors. (Matt Nordhoff, #377551)
+
+    - Fix serving branches over HTTP. (Matt Nordhoff, Jelmer Vernooij,
+      #380026)
+
+    - Install loggerhead as a bzr plugin by default (Jelmer Vernooij)
+
+    - Fix logging 404 Not Found responses (Matt Nordhoff, #381029)
+
 1.10 [22Dec2008]
 ---------------
 

=== modified file '__init__.py'
--- __init__.py	2009-05-06 18:48:16 +0000
+++ __init__.py	2009-06-03 23:21:52 +0000
@@ -38,51 +38,81 @@
 
     require_any_api(bzrlib, [(1, 11, 0), (1, 13, 0), (1, 15, 0)])
 
-    # TODO: This should provide a new type of server that can be used by bzr
-    # serve, maybe through a registry, rather than overriding the command.  Though
-    # maybe we should keep the wrapper to work with older bzr releases, at least
-    # for a bit.
-
     # NB: Normally plugins should lazily load almost everything, but this
     # seems reasonable to have in-line here: bzrlib.commands and options are
     # normally loaded, and the rest of loggerhead won't be loaded until serve
     # --http is run.
-        
-    import bzrlib.builtins
-    from bzrlib.commands import get_cmd_object, register_command
-    from bzrlib.option import Option
-
-    _original_command = get_cmd_object('serve')
-
+
+    # transport_server_registry was added in bzr 1.16. When we drop support for
+    # older releases, we can remove the code to override cmd_serve.
+
+    try:
+        from bzrlib.transport import transport_server_registry
+    except ImportError:
+        transport_server_registry = None
+
+    DEFAULT_HOST = '0.0.0.0'
     DEFAULT_PORT = 8080
-
-    class cmd_serve(bzrlib.builtins.cmd_serve):
-        __doc__ = _original_command.__doc__
-
-        takes_options = _original_command.takes_options + [
-            Option('http',
-                help='Run an http (Loggerhead) server to browse code, '
-                    'by default on port %s.' % DEFAULT_PORT)]
-
-        def run(self, *args, **kw):
-            if 'http' in kw:
-                # loggerhead internal code will try to 'import loggerhead', so
-                # let's put it on the path
-                import os.path, sys
-                sys.path.append(os.path.dirname(__file__))
-
-                from loggerhead.apps.transport import BranchesFromTransportRoot
-                from paste.httpexceptions import HTTPExceptionHandler
-                from paste.httpserver import serve
-                a = HTTPExceptionHandler(BranchesFromTransportRoot('.'))
-                port = kw.get('port', DEFAULT_PORT)
-                # port might be an int already...
-                if isinstance(port, basestring) and ':' in port:
-                    host, port = port.split(':')
+    HELP = ('Loggerhead, a web-based code viewer and server. (default port: %d)' %
+            (DEFAULT_PORT,))
+
+    def serve_http(transport, host=None, port=None, inet=None):
+        # loggerhead internal code will try to 'import loggerhead', so
+        # let's put it on the path if we can't find it in the existing path
+        try:
+            import loggerhead
+        except ImportError:
+            import os.path, sys
+            sys.path.append(os.path.dirname(__file__))
+
+        from loggerhead.apps.transport import BranchesFromTransportRoot
+        from loggerhead.config import LoggerheadConfig
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+        if host is None:
+            host = DEFAULT_HOST
+        if port is None:
+            port = DEFAULT_PORT
+        argv = ['--host', host, '--port', str(port), '--', transport.base]
+        if not transport.is_readonly():
+            argv.insert(0, '--allow-writes')
+        config = LoggerheadConfig(argv)
+        app = BranchesFromTransportRoot(transport, config)
+        app = HTTPExceptionHandler(app)
+        serve(app, host=host, port=port)
+
+    if transport_server_registry is not None:
+        transport_server_registry.register('http', serve_http, help=HELP)
+    else:
+        import bzrlib.builtins
+        from bzrlib.commands import get_cmd_object, register_command
+        from bzrlib.option import Option
+
+        _original_command = get_cmd_object('serve')
+
+        class cmd_serve(bzrlib.builtins.cmd_serve):
+            __doc__ = _original_command.__doc__
+
+            takes_options = _original_command.takes_options + [
+                Option('http', help=HELP)]
+
+            def run(self, *args, **kw):
+                if 'http' in kw:
+                    from bzrlib.transport import get_transport
+                    allow_writes = kw.get('allow_writes', False)
+                    path = kw.get('directory', '.')
+                    port = kw.get('port', DEFAULT_PORT)
+                    # port might be an int already...
+                    if isinstance(port, basestring) and ':' in port:
+                        host, port = port.split(':')
+                    else:
+                        host = DEFAULT_HOST
+                    if allow_writes:
+                        transport = get_transport(path)
+                    else:
+                        transport = get_transport('readonly+' + path)
+                    serve_http(transport, host, port)
                 else:
-                    host = '0.0.0.0'
-                serve(a, host=host, port=port)
-            else:
-                super(cmd_serve, self).run(*args, **kw)
+                    super(cmd_serve, self).run(*args, **kw)
 
-    register_command(cmd_serve)
+        register_command(cmd_serve)

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-05-13 11:34:09 +0000
+++ loggerhead/apps/branch.py	2009-05-27 16:21:06 +0000
@@ -123,7 +123,12 @@
             if public_branch is not None:
                 self.served_url = public_branch
             else:
-                self.served_url = self.url([])
+                # Loggerhead only supports serving .bzr/ on local branches, so
+                # we shouldn't suggest something that won't work.
+                if self.branch.base.startswith('file://'):
+                    self.served_url = self.url([])
+                else:
+                    self.served_url = None
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2009-05-04 20:28:50 +0000
+++ loggerhead/apps/transport.py	2009-05-25 01:03:25 +0000
@@ -82,8 +82,15 @@
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
         elif '/.bzr/' in environ['PATH_INFO']:
-            app = urlparser.make_static(None, self.transport)
-            return app(environ, start_response)
+            # TODO: Use something here that uses the transport API 
+            # rather than relying on the local filesystem API.
+            try:
+                path = urlutils.local_path_from_url(self.transport.base)
+            except errors.InvalidURL:
+                raise httpexceptions.HTTPNotFound()
+            else:
+                app = urlparser.make_static(None, path)
+                return app(environ, start_response)
         else:
             return BranchesFromTransportServer(
                 self.transport, self)(environ, start_response)

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2009-04-16 22:28:17 +0000
+++ loggerhead/config.py	2009-06-03 23:21:52 +0000
@@ -1,8 +1,23 @@
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
 '''Configuration tools for Loggerhead.'''
+
 from optparse import OptionParser
 import sys
 import tempfile
 
+from bzrlib import config
 
 _temporary_sql_dir = None
 
@@ -20,6 +35,7 @@
         log_folder=None,
         use_cdn=False,
         sql_dir=None,
+        allow_writes=False,
         )
     parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
                       help="Serve user directories as ~user.")
@@ -32,7 +48,7 @@
                       help="Host Loggerhead should listen on.")
     parser.add_option('--memory-profile', action='store_true',
                       dest='memory_profile',
-                      help='Profile the memory usage using heapy.')
+                      help='Profile the memory usage using Dozer.')
     parser.add_option("--prefix", dest="user_prefix",
                       help="Specify host prefix.")
     parser.add_option("--profile", action="store_true", dest="profile",
@@ -45,19 +61,23 @@
                       type=str, help="The directory to place log files in.")
     parser.add_option("--version", action="store_true", dest="show_version",
                       help="Print the software version and exit")
-    parser.add_option('--use-cdn', action='store_true',
+    parser.add_option("--use-cdn", action="store_true", dest="use_cdn",
                       help="Serve YUI from Yahoo!'s CDN")
-    parser.add_option('--cache-dir', dest='sql_dir',
+    parser.add_option("--cache-dir", dest="sql_dir",
                       help="The directory to place the SQL cache in")
+    parser.add_option('--allow-writes', action='store_true',
+                      help="Allow writing to the Bazaar server.")
     return parser
 
 
 class LoggerheadConfig(object):
     '''A configuration object.'''
 
-    def __init__(self):
+    def __init__(self, argv=None):
+        if argv is None:
+            argv = sys.argv[1:]
         self._parser = command_line_parser()
-        self._options, self._args = self._parser.parse_args(sys.argv[1:])
+        self._options, self._args = self._parser.parse_args(argv)
 
         sql_dir = self.get_option('sql_dir')
         if sql_dir is None:
@@ -65,19 +85,27 @@
         self.SQL_DIR = sql_dir
 
     def get_option(self, option):
-        '''Get an option from the options dict.'''
-        return getattr(self._options, option)
+        """Get the value for the config option, either 
+           from ~/.bazaar/bazaar.conf or from the command line.
+           All loggerhead-specific settings start with 'http_'"""
+        global_config = config.GlobalConfig().get_user_option('http_'+option)
+        cmd_config = getattr(self._options, option)
+        if global_config is not None and (
+                cmd_config is None or cmd_config is False):
+            return global_config
+        else:
+            return cmd_config
 
     def get_arg(self, index):
-        '''Get an arg from the arg list.'''
+        """Get an arg from the arg list."""
         return self._args[index]
 
     def print_help(self):
-        '''Wrapper around OptionParser.print_help.'''
+        """Wrapper around OptionParser.print_help."""
         return self._parser.print_help()
 
     @property
     def arg_count(self):
-        '''Return the number of args from the option parser.'''
+        """Return the number of args from the option parser."""
         return len(self._args)
 

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2009-03-18 19:34:07 +0000
+++ loggerhead/controllers/annotate_ui.py	2009-05-16 06:05:56 +0000
@@ -23,6 +23,7 @@
 
 import bzrlib.errors
 import bzrlib.textfile
+import bzrlib.osutils
 
 from paste.httpexceptions import HTTPBadRequest, HTTPServerError
 
@@ -48,8 +49,17 @@
 
         file_name = os.path.basename(self._history.get_path(revid, file_id))
 
-        try:
-            file_lines = tree.get_file_lines(file_id)
+        file_text = tree.get_file_text(file_id)
+        encoding = 'utf-8'
+        try:
+            file_text = file_text.decode(encoding)
+        except UnicodeDecodeError:
+            encoding = 'iso-8859-15'
+            file_text = file_text.decode(encoding)
+
+        file_lines = bzrlib.osutils.split_lines(file_text)
+
+        try:
             bzrlib.textfile.check_text_lines(file_lines)
         except bzrlib.errors.BinaryFile:
                 # bail out; this isn't displayable text
@@ -58,7 +68,7 @@
                                      change=util.Container())
         else:
             if highlight is not None:
-                hl_lines = highlight(file_name, ''.join(file_lines))
+                hl_lines = highlight(file_name, file_text, encoding)
                 hl_lines.extend([u''] * (len(file_lines) - len(hl_lines)))
             else:
                 hl_lines = map(cgi.escape, file_lines)

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-05-05 18:37:26 +0000
+++ loggerhead/controllers/directory_ui.py	2009-05-28 13:56:43 +0000
@@ -20,7 +20,7 @@
 import logging
 import stat
 
-from bzrlib import branch
+from bzrlib import branch, errors
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
@@ -73,7 +73,10 @@
             try:
                 b = branch.Branch.open_from_transport(self.transport.clone(d))
             except:
-                if not stat.S_ISDIR(self.transport.stat(d).st_mode):
+                try:
+                    if not stat.S_ISDIR(self.transport.stat(d).st_mode):
+                        continue
+                except errors.NoSuchFile:
                     continue
                 b = None
             dirs.append(DirEntry(d, parity, b))

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-03-19 19:04:40 +0000
+++ loggerhead/controllers/revision_ui.py	2009-04-29 19:55:16 +0000
@@ -95,6 +95,18 @@
 
         h.add_branch_nicks(change)
 
+        if '.' in change.revno:
+            # Walk "down" though the merge-sorted graph until we find a
+            # revision with merge depth 0: this is the revision that merged
+            # this one to mainline.
+            ri = self._history._rev_info
+            i = self._history._rev_indices[change.revid]
+            while ri[i][0][2] > 0:
+                i += 1
+            merged_in = ri[i][0][3]
+        else:
+            merged_in = None
+
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -116,6 +128,7 @@
             'filter_file_id': filter_file_id,
             'util': util,
             'history': h,
+            'merged_in': merged_in,
             'navigation': navigation,
             'query': query,
             'remember': remember,

=== modified file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	2009-03-06 13:19:38 +0000
+++ loggerhead/highlight.py	2009-05-16 06:03:47 +0000
@@ -24,20 +24,13 @@
 DEFAULT_PYGMENT_STYLE = 'colorful'
 
 
-def highlight(path, text, style=DEFAULT_PYGMENT_STYLE):
+def highlight(path, text, encoding, style=DEFAULT_PYGMENT_STYLE):
     """
     Returns a list of highlighted (i.e. HTML formatted) strings.
     """
 
     formatter = HtmlFormatter(style=style, nowrap=True, classprefix='pyg-')
 
-    encoding = 'utf-8'
-    try:
-        text = text.decode(encoding)
-    except UnicodeDecodeError:
-        encoding = 'iso-8859-15'
-        text = text.decode(encoding)
-
     try:
         lexer = guess_lexer_for_filename(path, text[:1024], encoding=encoding)
     except (ClassNotFound, ValueError):

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-06-06 09:09:55 +0000
+++ loggerhead/history.py	2009-06-06 09:16:19 +0000
@@ -31,13 +31,9 @@
 import bisect
 import datetime
 import logging
-import marshal
 import re
 import textwrap
 import threading
-import time
-import urllib
-from StringIO import StringIO
 
 from loggerhead import search
 from loggerhead import util
@@ -236,9 +232,6 @@
         parents of this revision.
     :ivar _rev_indices: A dictionary mapping each revision id to the index of
         the information about it in _rev_info.
-    :ivar _full_history: A list of all revision ids in the ancestry of the
-        branch, in merge-sorted order.  This is a bit silly, and shouldn't
-        really be stored on the instance...
     :ivar _revno_revid: A dictionary mapping stringified revnos to revision
         ids.
     """
@@ -271,19 +264,15 @@
             update_missed_caches()
 
         if self._rev_indices is not None:
-            self._full_history = []
             self._revno_revid = {}
             for ((_, revid, _, revno_str, _), _, _) in self._rev_info:
                 self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
         else:
-            self._full_history = []
             self._revno_revid = {}
             self._rev_indices = {}
             for ((seq, revid, _, revno_str, _), _, _) in self._rev_info:
                 self._rev_indices[revid] = seq
                 self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
 
     def __init__(self, branch, whole_history_data_cache, file_cache=None,
                  revinfo_disk_cache=None, cache_key=None):
@@ -327,7 +316,7 @@
         revid in revid_list.
         """
         if revid_list is None:
-            revid_list = self._full_history
+            revid_list = [r[0][1] for r in self._rev_info]
         revid_set = set(revid_list)
         revid = start_revid
 
@@ -359,9 +348,9 @@
             w = self._branch.repository.weave_store.get_weave(
                     file_id, self._branch.repository.get_transaction())
             w_revids = w.versions()
-            revids = [r for r in self._full_history if r in w_revids]
+            revids = [r for r in self._rev_indices if r in w_revids]
         except AttributeError:
-            possible_keys = [(file_id, revid) for revid in self._full_history]
+            possible_keys = [(file_id, revid) for revid in self._rev_indices]
             get_parent_map = self._branch.repository.texts.get_parent_map
             # We chunk the requests as this works better with GraphIndex.
             # See _filter_revisions_touching_file_id in bzrlib/log.py
@@ -691,6 +680,7 @@
             'comment': revision.message,
             'comment_clean': [util.html_clean(s) for s in message],
             'parents': revision.parent_ids,
+            'bugs': [bug.split()[0] for bug in revision.properties.get('bugs', '').splitlines()],
         }
         return util.Container(entry)
 

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-02-26 20:55:46 +0000
+++ loggerhead/static/css/diff.css	2009-05-30 17:40:05 +0000
@@ -31,7 +31,7 @@
     width:493px;
     margin:0px 0px 5px 0px;
 }
-.information, .committer, .timer, .mfrom, .mto, .revid {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
     clear:both;
     margin:0px;
     padding:0px 0px 2px 18px;
@@ -65,6 +65,14 @@
 .mto {
     background:url(../images/ico_mergeto.gif) no-repeat top left;
 }
+.bug {
+    padding:1px 0px 0px 18px;
+    margin:0px 10px 5px 0px;
+    float:left;
+    clear:none;
+    display:inline;
+    background:url(../images/ico_bug.png) no-repeat top left;
+}
 /**/
 #loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
     color:#0f6b97;

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-01-23 20:53:40 +0000
+++ loggerhead/static/css/files.css	2009-05-28 22:48:45 +0000
@@ -1,5 +1,5 @@
 /*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
 	float:left;
 	margin:0px 10px 5px 0px;
 	padding:0px 0px 2px 18px;
@@ -39,3 +39,6 @@
     width:100%;
     position:relative;
 }
+.bug {
+	background:url(../images/ico_bug.png) no-repeat top left;
+}

=== added file 'loggerhead/static/images/ico_bug.png'
Binary files loggerhead/static/images/ico_bug.png and loggerhead/static/images/ico_bug.png differ
=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-03-19 00:18:46 +0000
+++ loggerhead/templatefunctions.py	2009-04-28 03:01:54 +0000
@@ -57,12 +57,12 @@
 
 
 @templatefunc
-def revisioninfo(url, branch, entry, file_changes=None, currently_showing=None):
+def revisioninfo(url, branch, entry, file_changes=None, currently_showing=None, merged_in=None):
     from loggerhead import util
     return _pt('revisioninfo').expand(
         url=url, change=entry, branch=branch, util=util,
         file_changes=file_changes, currently_showing=currently_showing,
-        **templatefunctions)
+        merged_in=merged_in, **templatefunctions)
 
 
 @templatefunc

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-03-31 21:15:48 +0000
+++ loggerhead/templates/revision.pt	2009-04-28 03:01:54 +0000
@@ -96,7 +96,7 @@
           </ul>
         </tal:we-are-comparing>
 
-        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, file_changes, specific_path)" />
+        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, file_changes, specific_path, merged_in)" />
         <tal:specific-path condition="not:specific_path">
           <p class="expand show_if_js" id="expand_all"><a href="#">
               <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-03-19 00:18:46 +0000
+++ loggerhead/templates/revisioninfo.pt	2009-05-30 17:40:05 +0000
@@ -12,7 +12,16 @@
                                href python:url(['/revision', merge_point.revno], clear=1)"
                tal:content="python:revno_with_nick(merge_point)"></a>
         </div>
+        <div class="mto" tal:condition="merged_in">
+          This revision was merged to the branch mainline in
+          <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
+            <tal:b content="merged_in"/></a>.
+        </div>
         <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
+        <div class="clear"><!-- --></div>
+        <div class="bug" tal:repeat="bug change/bugs">
+            <a tal:attributes="href bug" tal:content="bug"></a>
+        </div>
         <div class="information" tal:content="structure python:util.fixed_width(change.comment)"></div>
     </div>
     <ul id="list-files" tal:condition="file_changes">

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-02-18 10:40:22 +0000
+++ loggerhead/tests/test_simple.py	2009-05-29 13:25:59 +0000
@@ -3,6 +3,7 @@
 
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.util.configobj.configobj import ConfigObj
+from bzrlib import config
 
 from loggerhead.apps.branch import BranchWSGIApp
 from paste.fixture import TestApp
@@ -118,3 +119,19 @@
         res = app.get('/files')
         res.mustcontain('No revisions!')
 
+#class TestGlobalConfig(BasicTests):
+#    """
+#    Test that global config settings are respected
+#    """
+
+#    def setUp(self):
+#        BasicTests.setUp(self)
+#        self.createBranch()
+#        config.GlobalConfig().set_user_option('http_version', 'True')
+
+#    def test_setting_respected(self):
+        #FIXME: Figure out how to test this properly
+#        app = self.setUpLoggerhead()
+#        res = app.get('/changes', status=200)
+
+

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2009-04-24 10:30:15 +0000
+++ loggerhead/wholehistory.py	2009-05-18 04:17:11 +0000
@@ -53,7 +53,6 @@
          graph.iter_ancestry([last_revid]) if value is not None))
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
-    _full_history = []
 
     _rev_info = []
     _rev_indices = {}
@@ -66,7 +65,6 @@
 
     for info in _merge_sort:
         seq, revid, merge_depth, revno, end_of_merge = info
-        _full_history.append(revid)
         revno_str = '.'.join(str(n) for n in revno)
         parents = _revision_graph[revid]
         _rev_indices[revid] = len(_rev_info)

=== modified file 'serve-branches'
--- serve-branches	2009-05-04 20:21:08 +0000
+++ serve-branches	2009-06-05 01:59:11 +0000
@@ -1,4 +1,7 @@
 #!/usr/bin/env python
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -51,7 +54,10 @@
 
     load_plugins()
 
-    transport = get_transport(path)
+    if config.get_option('allow_writes'):
+        transport = get_transport(path)
+    else:
+        transport = get_transport('readonly+' + path)
 
     if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
         print "--trunk-dir is only valid with --user-dirs"
@@ -89,7 +95,6 @@
 
     # setup_logging() #end
 
-    app = TransLogger(app, logger=logger)
     if config.get_option('profile'):
         from loggerhead.middleware.profile import LSProfMiddleware
         app = LSProfMiddleware(app)
@@ -124,6 +129,7 @@
 
     app = HTTPExceptionHandler(app)
     app = ErrorHandlerApp(app)
+    app = TransLogger(app, logger=logger)
 
     if not config.get_option('user_port'):
         port = '8080'

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2008-11-20 16:55:47 +0000
+++ setup.py	2009-06-02 22:10:00 +0000
@@ -34,7 +34,9 @@
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
-                "loggerhead/templates"],
+                "loggerhead/templates",
+                "bzrlib.plugins.loggerhead"],
+    package_dir={'bzrlib.plugins.loggerhead':'.'},
     package_data = {"loggerhead": ["templates/*.pt",
                                    "static/css/*.css",
                                    "static/javascript/*.js",

=== modified file 'start-loggerhead'
--- start-loggerhead	2009-02-19 01:24:20 +0000
+++ start-loggerhead	2009-06-03 23:09:47 +0000
@@ -13,7 +13,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""A script for starting the loggerhead process."""
+"""
+WARNING! This script has been deprecated and will go away soon!
+A script for starting the loggerhead process.
+"""
 
 
 import logging
@@ -37,6 +40,13 @@
 
 
 def main():
+    sys.stderr.write('\n\n')
+    sys.stderr.write('WARNING!!! This script has been deprecated by '
+                     'serve-branches, and will be removed in the next '
+                     'release. Please migrate to serve-branches and report '
+                     'any missing features.\n')
+    sys.stderr.write('\n\n')
+
     home = os.path.realpath(os.path.dirname(__file__))
     default_pidfile = os.path.join(home, 'loggerhead.pid')
     default_configfile = os.path.join(home, 'loggerhead.conf')

